# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.provider import Provider
from ..types.provider_type import ProviderType
from .raw_client import AsyncRawProvidersClient, RawProvidersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProvidersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProvidersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProvidersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        name: typing.Optional[str] = None,
        provider_type: typing.Optional[ProviderType] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Provider]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        name : typing.Optional[str]

        provider_type : typing.Optional[ProviderType]

        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Provider]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.list()
        """
        _response = self._raw_client.list(
            name=name, provider_type=provider_type, after=after, limit=limit, request_options=request_options
        )
        return _response.data

    def create(
        self,
        *,
        name: str,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Provider:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.create(
            name="name",
            provider_type="anthropic",
            api_key="api_key",
        )
        """
        _response = self._raw_client.create(
            name=name,
            provider_type=provider_type,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.delete(
            provider_id="provider_id",
        )
        """
        _response = self._raw_client.delete(provider_id, request_options=request_options)
        return _response.data

    def modify(
        self,
        provider_id: str,
        *,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Provider:
        """
        Update an existing custom provider

        Parameters
        ----------
        provider_id : str

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.modify(
            provider_id="provider_id",
            api_key="api_key",
        )
        """
        _response = self._raw_client.modify(
            provider_id,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data

    def check(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.check()
        """
        _response = self._raw_client.check(request_options=request_options)
        return _response.data

    def check_provider(
        self,
        *,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.providers.check_provider(
            provider_type="anthropic",
            api_key="api_key",
        )
        """
        _response = self._raw_client.check_provider(
            provider_type=provider_type,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data


class AsyncProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProvidersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProvidersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProvidersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        name: typing.Optional[str] = None,
        provider_type: typing.Optional[ProviderType] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Provider]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        name : typing.Optional[str]

        provider_type : typing.Optional[ProviderType]

        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Provider]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            name=name, provider_type=provider_type, after=after, limit=limit, request_options=request_options
        )
        return _response.data

    async def create(
        self,
        *,
        name: str,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Provider:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.create(
                name="name",
                provider_type="anthropic",
                api_key="api_key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            provider_type=provider_type,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[typing.Any]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.delete(
                provider_id="provider_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(provider_id, request_options=request_options)
        return _response.data

    async def modify(
        self,
        provider_id: str,
        *,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Provider:
        """
        Update an existing custom provider

        Parameters
        ----------
        provider_id : str

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Provider
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.modify(
                provider_id="provider_id",
                api_key="api_key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.modify(
            provider_id,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data

    async def check(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.check()


        asyncio.run(main())
        """
        _response = await self._raw_client.check(request_options=request_options)
        return _response.data

    async def check_provider(
        self,
        *,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.providers.check_provider(
                provider_type="anthropic",
                api_key="api_key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.check_provider(
            provider_type=provider_type,
            api_key=api_key,
            access_key=access_key,
            region=region,
            base_url=base_url,
            api_version=api_version,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ActionModel,
    ActionParametersModel,
    ActionResponseModel,
    AgentEnvironmentVariable,
    AgentFileAttachment,
    AgentFileSchema,
    AgentState,
    AgentStateResponseFormat,
    AgentStateToolRulesItem,
    AgentType,
    AppAuthScheme,
    AppAuthSchemeAuthMode,
    AppModel,
    AssistantMessage,
    AssistantMessageContent,
    Audio,
    AuthRequest,
    AuthResponse,
    AuthSchemeField,
    BadRequestErrorBody,
    Base64Image,
    BaseToolRuleSchema,
    BatchJob,
    Block,
    BlockSchema,
    BlockUpdate,
    BodyExportAgentSerialized,
    ChatCompletionAllowedToolChoiceParam,
    ChatCompletionAllowedToolsParam,
    ChatCompletionAllowedToolsParamMode,
    ChatCompletionAssistantMessageParam,
    ChatCompletionAssistantMessageParamContent,
    ChatCompletionAssistantMessageParamContentItem,
    ChatCompletionAssistantMessageParamToolCallsItem,
    ChatCompletionAudioParam,
    ChatCompletionAudioParamFormat,
    ChatCompletionAudioParamVoice,
    ChatCompletionContentPartImageParam,
    ChatCompletionContentPartInputAudioParam,
    ChatCompletionContentPartRefusalParam,
    ChatCompletionContentPartTextParam,
    ChatCompletionCustomToolParam,
    ChatCompletionDeveloperMessageParam,
    ChatCompletionDeveloperMessageParamContent,
    ChatCompletionFunctionCallOptionParam,
    ChatCompletionFunctionMessageParam,
    ChatCompletionFunctionToolParam,
    ChatCompletionMessageCustomToolCallParam,
    ChatCompletionMessageFunctionToolCallInput,
    ChatCompletionMessageFunctionToolCallOutput,
    ChatCompletionMessageFunctionToolCallParam,
    ChatCompletionNamedToolChoiceCustomParam,
    ChatCompletionNamedToolChoiceParam,
    ChatCompletionPredictionContentParam,
    ChatCompletionPredictionContentParamContent,
    ChatCompletionStreamOptionsParam,
    ChatCompletionSystemMessageParam,
    ChatCompletionSystemMessageParamContent,
    ChatCompletionToolMessageParam,
    ChatCompletionToolMessageParamContent,
    ChatCompletionUserMessageParam,
    ChatCompletionUserMessageParamContent,
    ChatCompletionUserMessageParamContentItem,
    ChildToolRule,
    ChildToolRuleSchema,
    CodeInput,
    CompletionCreateParamsNonStreaming,
    CompletionCreateParamsNonStreamingFunctionCall,
    CompletionCreateParamsNonStreamingMessagesItem,
    CompletionCreateParamsNonStreamingModalitiesItem,
    CompletionCreateParamsNonStreamingModel,
    CompletionCreateParamsNonStreamingReasoningEffort,
    CompletionCreateParamsNonStreamingResponseFormat,
    CompletionCreateParamsNonStreamingServiceTier,
    CompletionCreateParamsNonStreamingStop,
    CompletionCreateParamsNonStreamingToolChoice,
    CompletionCreateParamsNonStreamingToolsItem,
    CompletionCreateParamsNonStreamingVerbosity,
    CompletionCreateParamsStreaming,
    CompletionCreateParamsStreamingFunctionCall,
    CompletionCreateParamsStreamingMessagesItem,
    CompletionCreateParamsStreamingModalitiesItem,
    CompletionCreateParamsStreamingModel,
    CompletionCreateParamsStreamingReasoningEffort,
    CompletionCreateParamsStreamingResponseFormat,
    CompletionCreateParamsStreamingServiceTier,
    CompletionCreateParamsStreamingStop,
    CompletionCreateParamsStreamingToolChoice,
    CompletionCreateParamsStreamingToolsItem,
    CompletionCreateParamsStreamingVerbosity,
    ComponentsSchemasTextContent,
    ConditionalToolRule,
    ConditionalToolRuleSchema,
    ConflictErrorBody,
    ContextWindowOverview,
    ContinueToolRule,
    CoreMemoryBlockSchema,
    CreateBlock,
    CustomFormatGrammar,
    CustomFormatGrammarGrammar,
    CustomFormatGrammarGrammarSyntax,
    CustomFormatText,
    DuplicateFileHandling,
    DynamicManager,
    DynamicManagerUpdate,
    E2BSandboxConfig,
    EmbeddingConfig,
    EmbeddingConfigEmbeddingEndpointType,
    FeedbackType,
    File,
    FileAgentSchema,
    FileBlock,
    FileFile,
    FileMetadata,
    FileProcessingStatus,
    FileSchema,
    FileStats,
    Folder,
    FunctionCall,
    FunctionDefinitionInput,
    FunctionDefinitionOutput,
    FunctionOutput,
    FunctionTool,
    GenerateToolInput,
    GenerateToolOutput,
    Group,
    GroupSchema,
    GroupSchemaManagerConfig,
    Health,
    HiddenReasoningMessage,
    HiddenReasoningMessageState,
    HttpValidationError,
    Identity,
    IdentityProperty,
    IdentityPropertyType,
    IdentityPropertyValue,
    IdentityType,
    ImageContent,
    ImageContentSource,
    ImageUrl,
    ImageUrlDetail,
    ImportedAgentsResponse,
    InitToolRule,
    InputAudio,
    InputAudioFormat,
    Job,
    JobStatus,
    JobType,
    JsonObjectResponseFormat,
    JsonSchema,
    JsonSchemaResponseFormat,
    LettaBatchMessages,
    LettaBatchRequest,
    LettaImage,
    LettaMessageContentUnion,
    LettaMessageUnion,
    LettaPing,
    LettaRequest,
    LettaRequestConfig,
    LettaResponse,
    LettaSchemasAgentFileAgentSchema,
    LettaSchemasAgentFileAgentSchemaResponseFormat,
    LettaSchemasAgentFileAgentSchemaToolRulesItem,
    LettaSchemasAgentFileMessageSchema,
    LettaSchemasAgentFileMessageSchemaContent,
    LettaSchemasAgentFileToolSchema,
    LettaSerializeSchemasPydanticAgentSchemaAgentSchema,
    LettaSerializeSchemasPydanticAgentSchemaAgentSchemaToolRulesItem,
    LettaSerializeSchemasPydanticAgentSchemaMessageSchema,
    LettaSerializeSchemasPydanticAgentSchemaToolSchema,
    LettaStopReason,
    LettaStreamingRequest,
    LettaUsageStatistics,
    LettaUserMessageContentUnion,
    LlmConfig,
    LlmConfigCompatibilityType,
    LlmConfigModelEndpointType,
    LlmConfigReasoningEffort,
    LlmConfigVerbosity,
    LocalSandboxConfig,
    ManagerType,
    MaxCountPerStepToolRule,
    MaxCountPerStepToolRuleSchema,
    McpServerSchema,
    McpServerType,
    McpTool,
    McpToolExecuteRequest,
    McpToolHealth,
    Memory,
    Message,
    MessageContentItem,
    MessageCreate,
    MessageCreateContent,
    MessageCreateRole,
    MessageRole,
    MessageType,
    ModalSandboxConfig,
    ModalSandboxConfigLanguage,
    NotFoundErrorBody,
    NpmRequirement,
    OmittedReasoningContent,
    OpenaiTypesChatChatCompletionCustomToolParamCustom,
    OpenaiTypesChatChatCompletionCustomToolParamCustomFormat,
    OpenaiTypesChatChatCompletionMessageCustomToolCallParamCustom,
    OpenaiTypesChatChatCompletionMessageFunctionToolCallFunction,
    OpenaiTypesChatChatCompletionMessageFunctionToolCallParamFunction,
    OpenaiTypesChatChatCompletionNamedToolChoiceCustomParamCustom,
    OpenaiTypesChatChatCompletionNamedToolChoiceParamFunction,
    OpenaiTypesChatCompletionCreateParamsFunction,
    Organization,
    OrganizationCreate,
    OrganizationSourcesStats,
    OrganizationUpdate,
    PaginatedAgentFiles,
    ParameterProperties,
    ParametersSchema,
    ParentToolRule,
    Passage,
    PaymentRequiredErrorBody,
    PipRequirement,
    Provider,
    ProviderCategory,
    ProviderTrace,
    ProviderType,
    ReasoningContent,
    ReasoningMessage,
    ReasoningMessageSource,
    RedactedReasoningContent,
    RequiredBeforeExitToolRule,
    ResponseFormatJsonObject,
    ResponseFormatJsonSchema,
    ResponseFormatText,
    RoundRobinManager,
    RoundRobinManagerUpdate,
    Run,
    SandboxConfig,
    SandboxConfigCreate,
    SandboxConfigCreateConfig,
    SandboxConfigUpdate,
    SandboxConfigUpdateConfig,
    SandboxEnvironmentVariable,
    SandboxEnvironmentVariableCreate,
    SandboxEnvironmentVariableUpdate,
    SandboxType,
    SleeptimeManager,
    SleeptimeManagerUpdate,
    Source,
    SourceCreate,
    SourceSchema,
    SourceStats,
    SourceUpdate,
    SseServerConfig,
    StdioServerConfig,
    Step,
    StepFeedback,
    StepMetrics,
    StepStatus,
    StopReasonType,
    StreamableHttpServerConfig,
    SupervisorManager,
    SupervisorManagerUpdate,
    SystemMessage,
    TagSchema,
    TerminalToolRule,
    TextContent,
    TextResponseFormat,
    Tool,
    ToolAnnotations,
    ToolCall,
    ToolCallContent,
    ToolCallDelta,
    ToolCallMessage,
    ToolCallMessageToolCall,
    ToolCreate,
    ToolEnvVarSchema,
    ToolJsonSchema,
    ToolReturn,
    ToolReturnContent,
    ToolReturnMessage,
    ToolReturnMessageStatus,
    ToolReturnStatus,
    ToolType,
    UpdateAssistantMessage,
    UpdateAssistantMessageContent,
    UpdateReasoningMessage,
    UpdateSsemcpServer,
    UpdateStdioMcpServer,
    UpdateStreamableHttpmcpServer,
    UpdateSystemMessage,
    UpdateUserMessage,
    UpdateUserMessageContent,
    UrlImage,
    UsageStatistics,
    UsageStatisticsCompletionTokenDetails,
    UsageStatisticsPromptTokenDetails,
    User,
    UserCreate,
    UserMessage,
    UserMessageContent,
    UserUpdate,
    ValidationError,
    ValidationErrorLocItem,
    VoiceSleeptimeManager,
    VoiceSleeptimeManagerUpdate,
    WebSearchOptions,
    WebSearchOptionsSearchContextSize,
    WebSearchOptionsUserLocation,
    WebSearchOptionsUserLocationApproximate,
)
from .errors import BadRequestError, ConflictError, NotFoundError, PaymentRequiredError, UnprocessableEntityError
from . import (
    agents,
    batches,
    blocks,
    client_side_access_tokens,
    embedding_models,
    folders,
    groups,
    health,
    identities,
    jobs,
    messages,
    models,
    projects,
    providers,
    runs,
    sources,
    steps,
    tags,
    telemetry,
    templates,
    tools,
    voice,
)
from .agents import (
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItemField,
    AgentsSearchRequestSearchItemOne,
    AgentsSearchRequestSearchItemOneOperator,
    AgentsSearchRequestSearchItemThree,
    AgentsSearchRequestSearchItemTwo,
    AgentsSearchRequestSearchItemZero,
    AgentsSearchRequestSortBy,
    AgentsSearchResponse,
    CreateAgentRequestResponseFormat,
    CreateAgentRequestToolRulesItem,
    UpdateAgentResponseFormat,
    UpdateAgentToolRulesItem,
)
from .client import AsyncLetta, Letta
from .client_side_access_tokens import (
    ClientSideAccessTokensCreateRequestPolicyItem,
    ClientSideAccessTokensCreateRequestPolicyItemAccessItem,
    ClientSideAccessTokensCreateResponse,
    ClientSideAccessTokensCreateResponsePolicy,
    ClientSideAccessTokensCreateResponsePolicyDataItem,
    ClientSideAccessTokensCreateResponsePolicyDataItemAccessItem,
    ClientSideAccessTokensListClientSideAccessTokensResponse,
    ClientSideAccessTokensListClientSideAccessTokensResponseTokensItem,
    ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicy,
    ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicyDataItem,
    ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicyDataItemAccessItem,
)
from .environment import LettaEnvironment
from .groups import GroupCreateManagerConfig, GroupUpdateManagerConfig
from .projects import ProjectsListResponse, ProjectsListResponseProjectsItem
from .runs import LettaStreamingResponse
from .steps import StepsListRequestFeedback
from .templates import (
    TemplatesCreateTemplateResponse,
    TemplatesDeleteTemplateResponse,
    TemplatesForkTemplateResponse,
    TemplatesGetTemplateSnapshotResponse,
    TemplatesGetTemplateSnapshotResponseAgentsItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariables,
    TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemProperties,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolVariables,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolVariablesDataItem,
    TemplatesGetTemplateSnapshotResponseBlocksItem,
    TemplatesGetTemplateSnapshotResponseConfiguration,
    TemplatesGetTemplateSnapshotResponseType,
    TemplatesListRequestSortBy,
    TemplatesListResponse,
    TemplatesListResponseTemplatesItem,
    TemplatesListTemplateVersionsResponse,
    TemplatesListTemplateVersionsResponseVersionsItem,
    TemplatesRenameTemplateResponse,
    TemplatesSaveTemplateVersionResponse,
)
from .tools import (
    AddMcpServerRequest,
    AddMcpServerResponseItem,
    ConnectMcpServerRequest,
    ConnectMcpServerResponseEvent,
    DeleteMcpServerResponseItem,
    ListMcpServersResponseValue,
    StreamingResponse,
    TestMcpServerRequest,
    UpdateMcpServerRequest,
    UpdateMcpServerResponse,
)
from .version import __version__

__all__ = [
    "ActionModel",
    "ActionParametersModel",
    "ActionResponseModel",
    "AddMcpServerRequest",
    "AddMcpServerResponseItem",
    "AgentEnvironmentVariable",
    "AgentFileAttachment",
    "AgentFileSchema",
    "AgentState",
    "AgentStateResponseFormat",
    "AgentStateToolRulesItem",
    "AgentType",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemField",
    "AgentsSearchRequestSearchItemOne",
    "AgentsSearchRequestSearchItemOneOperator",
    "AgentsSearchRequestSearchItemThree",
    "AgentsSearchRequestSearchItemTwo",
    "AgentsSearchRequestSearchItemZero",
    "AgentsSearchRequestSortBy",
    "AgentsSearchResponse",
    "AppAuthScheme",
    "AppAuthSchemeAuthMode",
    "AppModel",
    "AssistantMessage",
    "AssistantMessageContent",
    "AsyncLetta",
    "Audio",
    "AuthRequest",
    "AuthResponse",
    "AuthSchemeField",
    "BadRequestError",
    "BadRequestErrorBody",
    "Base64Image",
    "BaseToolRuleSchema",
    "BatchJob",
    "Block",
    "BlockSchema",
    "BlockUpdate",
    "BodyExportAgentSerialized",
    "ChatCompletionAllowedToolChoiceParam",
    "ChatCompletionAllowedToolsParam",
    "ChatCompletionAllowedToolsParamMode",
    "ChatCompletionAssistantMessageParam",
    "ChatCompletionAssistantMessageParamContent",
    "ChatCompletionAssistantMessageParamContentItem",
    "ChatCompletionAssistantMessageParamToolCallsItem",
    "ChatCompletionAudioParam",
    "ChatCompletionAudioParamFormat",
    "ChatCompletionAudioParamVoice",
    "ChatCompletionContentPartImageParam",
    "ChatCompletionContentPartInputAudioParam",
    "ChatCompletionContentPartRefusalParam",
    "ChatCompletionContentPartTextParam",
    "ChatCompletionCustomToolParam",
    "ChatCompletionDeveloperMessageParam",
    "ChatCompletionDeveloperMessageParamContent",
    "ChatCompletionFunctionCallOptionParam",
    "ChatCompletionFunctionMessageParam",
    "ChatCompletionFunctionToolParam",
    "ChatCompletionMessageCustomToolCallParam",
    "ChatCompletionMessageFunctionToolCallInput",
    "ChatCompletionMessageFunctionToolCallOutput",
    "ChatCompletionMessageFunctionToolCallParam",
    "ChatCompletionNamedToolChoiceCustomParam",
    "ChatCompletionNamedToolChoiceParam",
    "ChatCompletionPredictionContentParam",
    "ChatCompletionPredictionContentParamContent",
    "ChatCompletionStreamOptionsParam",
    "ChatCompletionSystemMessageParam",
    "ChatCompletionSystemMessageParamContent",
    "ChatCompletionToolMessageParam",
    "ChatCompletionToolMessageParamContent",
    "ChatCompletionUserMessageParam",
    "ChatCompletionUserMessageParamContent",
    "ChatCompletionUserMessageParamContentItem",
    "ChildToolRule",
    "ChildToolRuleSchema",
    "ClientSideAccessTokensCreateRequestPolicyItem",
    "ClientSideAccessTokensCreateRequestPolicyItemAccessItem",
    "ClientSideAccessTokensCreateResponse",
    "ClientSideAccessTokensCreateResponsePolicy",
    "ClientSideAccessTokensCreateResponsePolicyDataItem",
    "ClientSideAccessTokensCreateResponsePolicyDataItemAccessItem",
    "ClientSideAccessTokensListClientSideAccessTokensResponse",
    "ClientSideAccessTokensListClientSideAccessTokensResponseTokensItem",
    "ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicy",
    "ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicyDataItem",
    "ClientSideAccessTokensListClientSideAccessTokensResponseTokensItemPolicyDataItemAccessItem",
    "CodeInput",
    "CompletionCreateParamsNonStreaming",
    "CompletionCreateParamsNonStreamingFunctionCall",
    "CompletionCreateParamsNonStreamingMessagesItem",
    "CompletionCreateParamsNonStreamingModalitiesItem",
    "CompletionCreateParamsNonStreamingModel",
    "CompletionCreateParamsNonStreamingReasoningEffort",
    "CompletionCreateParamsNonStreamingResponseFormat",
    "CompletionCreateParamsNonStreamingServiceTier",
    "CompletionCreateParamsNonStreamingStop",
    "CompletionCreateParamsNonStreamingToolChoice",
    "CompletionCreateParamsNonStreamingToolsItem",
    "CompletionCreateParamsNonStreamingVerbosity",
    "CompletionCreateParamsStreaming",
    "CompletionCreateParamsStreamingFunctionCall",
    "CompletionCreateParamsStreamingMessagesItem",
    "CompletionCreateParamsStreamingModalitiesItem",
    "CompletionCreateParamsStreamingModel",
    "CompletionCreateParamsStreamingReasoningEffort",
    "CompletionCreateParamsStreamingResponseFormat",
    "CompletionCreateParamsStreamingServiceTier",
    "CompletionCreateParamsStreamingStop",
    "CompletionCreateParamsStreamingToolChoice",
    "CompletionCreateParamsStreamingToolsItem",
    "CompletionCreateParamsStreamingVerbosity",
    "ComponentsSchemasTextContent",
    "ConditionalToolRule",
    "ConditionalToolRuleSchema",
    "ConflictError",
    "ConflictErrorBody",
    "ConnectMcpServerRequest",
    "ConnectMcpServerResponseEvent",
    "ContextWindowOverview",
    "ContinueToolRule",
    "CoreMemoryBlockSchema",
    "CreateAgentRequestResponseFormat",
    "CreateAgentRequestToolRulesItem",
    "CreateBlock",
    "CustomFormatGrammar",
    "CustomFormatGrammarGrammar",
    "CustomFormatGrammarGrammarSyntax",
    "CustomFormatText",
    "DeleteMcpServerResponseItem",
    "DuplicateFileHandling",
    "DynamicManager",
    "DynamicManagerUpdate",
    "E2BSandboxConfig",
    "EmbeddingConfig",
    "EmbeddingConfigEmbeddingEndpointType",
    "FeedbackType",
    "File",
    "FileAgentSchema",
    "FileBlock",
    "FileFile",
    "FileMetadata",
    "FileProcessingStatus",
    "FileSchema",
    "FileStats",
    "Folder",
    "FunctionCall",
    "FunctionDefinitionInput",
    "FunctionDefinitionOutput",
    "FunctionOutput",
    "FunctionTool",
    "GenerateToolInput",
    "GenerateToolOutput",
    "Group",
    "GroupCreateManagerConfig",
    "GroupSchema",
    "GroupSchemaManagerConfig",
    "GroupUpdateManagerConfig",
    "Health",
    "HiddenReasoningMessage",
    "HiddenReasoningMessageState",
    "HttpValidationError",
    "Identity",
    "IdentityProperty",
    "IdentityPropertyType",
    "IdentityPropertyValue",
    "IdentityType",
    "ImageContent",
    "ImageContentSource",
    "ImageUrl",
    "ImageUrlDetail",
    "ImportedAgentsResponse",
    "InitToolRule",
    "InputAudio",
    "InputAudioFormat",
    "Job",
    "JobStatus",
    "JobType",
    "JsonObjectResponseFormat",
    "JsonSchema",
    "JsonSchemaResponseFormat",
    "Letta",
    "LettaBatchMessages",
    "LettaBatchRequest",
    "LettaEnvironment",
    "LettaImage",
    "LettaMessageContentUnion",
    "LettaMessageUnion",
    "LettaPing",
    "LettaRequest",
    "LettaRequestConfig",
    "LettaResponse",
    "LettaSchemasAgentFileAgentSchema",
    "LettaSchemasAgentFileAgentSchemaResponseFormat",
    "LettaSchemasAgentFileAgentSchemaToolRulesItem",
    "LettaSchemasAgentFileMessageSchema",
    "LettaSchemasAgentFileMessageSchemaContent",
    "LettaSchemasAgentFileToolSchema",
    "LettaSerializeSchemasPydanticAgentSchemaAgentSchema",
    "LettaSerializeSchemasPydanticAgentSchemaAgentSchemaToolRulesItem",
    "LettaSerializeSchemasPydanticAgentSchemaMessageSchema",
    "LettaSerializeSchemasPydanticAgentSchemaToolSchema",
    "LettaStopReason",
    "LettaStreamingRequest",
    "LettaStreamingResponse",
    "LettaUsageStatistics",
    "LettaUserMessageContentUnion",
    "ListMcpServersResponseValue",
    "LlmConfig",
    "LlmConfigCompatibilityType",
    "LlmConfigModelEndpointType",
    "LlmConfigReasoningEffort",
    "LlmConfigVerbosity",
    "LocalSandboxConfig",
    "ManagerType",
    "MaxCountPerStepToolRule",
    "MaxCountPerStepToolRuleSchema",
    "McpServerSchema",
    "McpServerType",
    "McpTool",
    "McpToolExecuteRequest",
    "McpToolHealth",
    "Memory",
    "Message",
    "MessageContentItem",
    "MessageCreate",
    "MessageCreateContent",
    "MessageCreateRole",
    "MessageRole",
    "MessageType",
    "ModalSandboxConfig",
    "ModalSandboxConfigLanguage",
    "NotFoundError",
    "NotFoundErrorBody",
    "NpmRequirement",
    "OmittedReasoningContent",
    "OpenaiTypesChatChatCompletionCustomToolParamCustom",
    "OpenaiTypesChatChatCompletionCustomToolParamCustomFormat",
    "OpenaiTypesChatChatCompletionMessageCustomToolCallParamCustom",
    "OpenaiTypesChatChatCompletionMessageFunctionToolCallFunction",
    "OpenaiTypesChatChatCompletionMessageFunctionToolCallParamFunction",
    "OpenaiTypesChatChatCompletionNamedToolChoiceCustomParamCustom",
    "OpenaiTypesChatChatCompletionNamedToolChoiceParamFunction",
    "OpenaiTypesChatCompletionCreateParamsFunction",
    "Organization",
    "OrganizationCreate",
    "OrganizationSourcesStats",
    "OrganizationUpdate",
    "PaginatedAgentFiles",
    "ParameterProperties",
    "ParametersSchema",
    "ParentToolRule",
    "Passage",
    "PaymentRequiredError",
    "PaymentRequiredErrorBody",
    "PipRequirement",
    "ProjectsListResponse",
    "ProjectsListResponseProjectsItem",
    "Provider",
    "ProviderCategory",
    "ProviderTrace",
    "ProviderType",
    "ReasoningContent",
    "ReasoningMessage",
    "ReasoningMessageSource",
    "RedactedReasoningContent",
    "RequiredBeforeExitToolRule",
    "ResponseFormatJsonObject",
    "ResponseFormatJsonSchema",
    "ResponseFormatText",
    "RoundRobinManager",
    "RoundRobinManagerUpdate",
    "Run",
    "SandboxConfig",
    "SandboxConfigCreate",
    "SandboxConfigCreateConfig",
    "SandboxConfigUpdate",
    "SandboxConfigUpdateConfig",
    "SandboxEnvironmentVariable",
    "SandboxEnvironmentVariableCreate",
    "SandboxEnvironmentVariableUpdate",
    "SandboxType",
    "SleeptimeManager",
    "SleeptimeManagerUpdate",
    "Source",
    "SourceCreate",
    "SourceSchema",
    "SourceStats",
    "SourceUpdate",
    "SseServerConfig",
    "StdioServerConfig",
    "Step",
    "StepFeedback",
    "StepMetrics",
    "StepStatus",
    "StepsListRequestFeedback",
    "StopReasonType",
    "StreamableHttpServerConfig",
    "StreamingResponse",
    "SupervisorManager",
    "SupervisorManagerUpdate",
    "SystemMessage",
    "TagSchema",
    "TemplatesCreateTemplateResponse",
    "TemplatesDeleteTemplateResponse",
    "TemplatesForkTemplateResponse",
    "TemplatesGetTemplateSnapshotResponse",
    "TemplatesGetTemplateSnapshotResponseAgentsItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariables",
    "TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemProperties",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolVariables",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolVariablesDataItem",
    "TemplatesGetTemplateSnapshotResponseBlocksItem",
    "TemplatesGetTemplateSnapshotResponseConfiguration",
    "TemplatesGetTemplateSnapshotResponseType",
    "TemplatesListRequestSortBy",
    "TemplatesListResponse",
    "TemplatesListResponseTemplatesItem",
    "TemplatesListTemplateVersionsResponse",
    "TemplatesListTemplateVersionsResponseVersionsItem",
    "TemplatesRenameTemplateResponse",
    "TemplatesSaveTemplateVersionResponse",
    "TerminalToolRule",
    "TestMcpServerRequest",
    "TextContent",
    "TextResponseFormat",
    "Tool",
    "ToolAnnotations",
    "ToolCall",
    "ToolCallContent",
    "ToolCallDelta",
    "ToolCallMessage",
    "ToolCallMessageToolCall",
    "ToolCreate",
    "ToolEnvVarSchema",
    "ToolJsonSchema",
    "ToolReturn",
    "ToolReturnContent",
    "ToolReturnMessage",
    "ToolReturnMessageStatus",
    "ToolReturnStatus",
    "ToolType",
    "UnprocessableEntityError",
    "UpdateAgentResponseFormat",
    "UpdateAgentToolRulesItem",
    "UpdateAssistantMessage",
    "UpdateAssistantMessageContent",
    "UpdateMcpServerRequest",
    "UpdateMcpServerResponse",
    "UpdateReasoningMessage",
    "UpdateSsemcpServer",
    "UpdateStdioMcpServer",
    "UpdateStreamableHttpmcpServer",
    "UpdateSystemMessage",
    "UpdateUserMessage",
    "UpdateUserMessageContent",
    "UrlImage",
    "UsageStatistics",
    "UsageStatisticsCompletionTokenDetails",
    "UsageStatisticsPromptTokenDetails",
    "User",
    "UserCreate",
    "UserMessage",
    "UserMessageContent",
    "UserUpdate",
    "ValidationError",
    "ValidationErrorLocItem",
    "VoiceSleeptimeManager",
    "VoiceSleeptimeManagerUpdate",
    "WebSearchOptions",
    "WebSearchOptionsSearchContextSize",
    "WebSearchOptionsUserLocation",
    "WebSearchOptionsUserLocationApproximate",
    "__version__",
    "agents",
    "batches",
    "blocks",
    "client_side_access_tokens",
    "embedding_models",
    "folders",
    "groups",
    "health",
    "identities",
    "jobs",
    "messages",
    "models",
    "projects",
    "providers",
    "runs",
    "sources",
    "steps",
    "tags",
    "telemetry",
    "templates",
    "tools",
    "voice",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .app_auth_scheme import AppAuthScheme


class AppModel(UncheckedBaseModel):
    """
    App data model.
    """

    name: str
    key: str
    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")]
    description: str
    categories: typing.List[str]
    meta: typing.Dict[str, typing.Optional[typing.Any]]
    logo: typing.Optional[str] = None
    docs: typing.Optional[str] = None
    group: typing.Optional[str] = None
    status: typing.Optional[str] = None
    enabled: typing.Optional[bool] = None
    no_auth: typing.Optional[bool] = None
    auth_schemes: typing.Optional[typing.List[AppAuthScheme]] = None
    test_connectors: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]],
        FieldMetadata(alias="testConnectors"),
    ] = None
    documentation_doc_text: typing.Optional[str] = None
    configuration_docs_text: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawTemplatesClient, RawTemplatesClient
from .types.templates_migrate_response import TemplatesMigrateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTemplatesClient
        """
        return self._raw_client

    def migrate(
        self,
        agent_id: str,
        *,
        to_template: str,
        preserve_core_memories: bool,
        preserve_tool_variables: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesMigrateResponse:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Migrate an agent to a new versioned agent template.

        Parameters
        ----------
        agent_id : str

        to_template : str

        preserve_core_memories : bool

        preserve_tool_variables : typing.Optional[bool]
            If true, preserves the existing agent's tool environment variables instead of using the template's variables

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesMigrateResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.agents.templates.migrate(
            agent_id="agent_id",
            to_template="to_template",
            preserve_core_memories=True,
        )
        """
        _response = self._raw_client.migrate(
            agent_id,
            to_template=to_template,
            preserve_core_memories=preserve_core_memories,
            preserve_tool_variables=preserve_tool_variables,
            request_options=request_options,
        )
        return _response.data

    def create(self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Creates a template from an agent.

        Parameters
        ----------
        agent_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.agents.templates.create(
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.create(agent_id, request_options=request_options)
        return _response.data

    def create_version(self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Creates a new version of the template version of the agent.

        Parameters
        ----------
        agent_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.agents.templates.create_version(
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.create_version(agent_id, request_options=request_options)
        return _response.data


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTemplatesClient
        """
        return self._raw_client

    async def migrate(
        self,
        agent_id: str,
        *,
        to_template: str,
        preserve_core_memories: bool,
        preserve_tool_variables: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesMigrateResponse:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Migrate an agent to a new versioned agent template.

        Parameters
        ----------
        agent_id : str

        to_template : str

        preserve_core_memories : bool

        preserve_tool_variables : typing.Optional[bool]
            If true, preserves the existing agent's tool environment variables instead of using the template's variables

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesMigrateResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.templates.migrate(
                agent_id="agent_id",
                to_template="to_template",
                preserve_core_memories=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.migrate(
            agent_id,
            to_template=to_template,
            preserve_core_memories=preserve_core_memories,
            preserve_tool_variables=preserve_tool_variables,
            request_options=request_options,
        )
        return _response.data

    async def create(self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Creates a template from an agent.

        Parameters
        ----------
        agent_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.templates.create(
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(agent_id, request_options=request_options)
        return _response.data

    async def create_version(self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        <Note>This endpoint is only available on Letta Cloud.</Note>

        Creates a new version of the template version of the agent.

        Parameters
        ----------
        agent_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.templates.create_version(
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_version(agent_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_content_part_text_param import ChatCompletionContentPartTextParam

ChatCompletionDeveloperMessageParamContent = typing.Union[str, typing.List[ChatCompletionContentPartTextParam]]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    TemplatesCreateTemplateResponse,
    TemplatesDeleteTemplateResponse,
    TemplatesForkTemplateResponse,
    TemplatesGetTemplateSnapshotResponse,
    TemplatesGetTemplateSnapshotResponseAgentsItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariables,
    TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemProperties,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItem,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolVariables,
    TemplatesGetTemplateSnapshotResponseAgentsItemToolVariablesDataItem,
    TemplatesGetTemplateSnapshotResponseBlocksItem,
    TemplatesGetTemplateSnapshotResponseConfiguration,
    TemplatesGetTemplateSnapshotResponseType,
    TemplatesListRequestSortBy,
    TemplatesListResponse,
    TemplatesListResponseTemplatesItem,
    TemplatesListTemplateVersionsResponse,
    TemplatesListTemplateVersionsResponseVersionsItem,
    TemplatesRenameTemplateResponse,
    TemplatesSaveTemplateVersionResponse,
)
from . import agents
from .agents import (
    AgentsCreateRequestInitialMessageSequenceItem,
    AgentsCreateRequestInitialMessageSequenceItemRole,
    AgentsCreateResponse,
)

__all__ = [
    "AgentsCreateRequestInitialMessageSequenceItem",
    "AgentsCreateRequestInitialMessageSequenceItemRole",
    "AgentsCreateResponse",
    "TemplatesCreateTemplateResponse",
    "TemplatesDeleteTemplateResponse",
    "TemplatesForkTemplateResponse",
    "TemplatesGetTemplateSnapshotResponse",
    "TemplatesGetTemplateSnapshotResponseAgentsItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariables",
    "TemplatesGetTemplateSnapshotResponseAgentsItemMemoryVariablesDataItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemProperties",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItem",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemChildOutputMapping",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFive",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemFour",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemMaxCountLimit",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemOne",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemSeven",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemTwo",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolRulesItemZero",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolVariables",
    "TemplatesGetTemplateSnapshotResponseAgentsItemToolVariablesDataItem",
    "TemplatesGetTemplateSnapshotResponseBlocksItem",
    "TemplatesGetTemplateSnapshotResponseConfiguration",
    "TemplatesGetTemplateSnapshotResponseType",
    "TemplatesListRequestSortBy",
    "TemplatesListResponse",
    "TemplatesListResponseTemplatesItem",
    "TemplatesListTemplateVersionsResponse",
    "TemplatesListTemplateVersionsResponseVersionsItem",
    "TemplatesRenameTemplateResponse",
    "TemplatesSaveTemplateVersionResponse",
    "agents",
]



# This file was auto-generated by Fern from our API Definition.

import typing

ProviderType = typing.Union[
    typing.Literal[
        "anthropic",
        "azure",
        "bedrock",
        "cerebras",
        "deepseek",
        "google_ai",
        "google_vertex",
        "groq",
        "hugging-face",
        "letta",
        "lmstudio_openai",
        "mistral",
        "ollama",
        "openai",
        "together",
        "vllm",
        "xai",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.provider import Provider
from ..types.provider_type import ProviderType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        name: typing.Optional[str] = None,
        provider_type: typing.Optional[ProviderType] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[Provider]]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        name : typing.Optional[str]

        provider_type : typing.Optional[ProviderType]

        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Provider]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="GET",
            params={
                "name": name,
                "provider_type": provider_type,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Provider],
                    construct_type(
                        type_=typing.List[Provider],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Provider]:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Provider]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="POST",
            json={
                "name": name,
                "provider_type": provider_type,
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/providers/{jsonable_encoder(provider_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def modify(
        self,
        provider_id: str,
        *,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Provider]:
        """
        Update an existing custom provider

        Parameters
        ----------
        provider_id : str

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Provider]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/providers/{jsonable_encoder(provider_id)}",
            method="PATCH",
            json={
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def check(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/check",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def check_provider(
        self,
        *,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/providers/check",
            method="POST",
            json={
                "provider_type": provider_type,
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        name: typing.Optional[str] = None,
        provider_type: typing.Optional[ProviderType] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[Provider]]:
        """
        Get a list of all custom providers in the database

        Parameters
        ----------
        name : typing.Optional[str]

        provider_type : typing.Optional[ProviderType]

        after : typing.Optional[str]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Provider]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="GET",
            params={
                "name": name,
                "provider_type": provider_type,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[Provider],
                    construct_type(
                        type_=typing.List[Provider],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Provider]:
        """
        Create a new custom provider

        Parameters
        ----------
        name : str
            The name of the provider.

        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Provider]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/",
            method="POST",
            json={
                "name": name,
                "provider_type": provider_type,
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, provider_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Delete an existing custom provider

        Parameters
        ----------
        provider_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/providers/{jsonable_encoder(provider_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def modify(
        self,
        provider_id: str,
        *,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Provider]:
        """
        Update an existing custom provider

        Parameters
        ----------
        provider_id : str

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Provider]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/providers/{jsonable_encoder(provider_id)}",
            method="PATCH",
            json={
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Provider,
                    construct_type(
                        type_=Provider,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def check(self, *, request_options: typing.Optional[RequestOptions] = None) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/check",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def check_provider(
        self,
        *,
        provider_type: ProviderType,
        api_key: str,
        access_key: typing.Optional[str] = OMIT,
        region: typing.Optional[str] = OMIT,
        base_url: typing.Optional[str] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        provider_type : ProviderType
            The type of the provider.

        api_key : str
            API key or secret key used for requests to the provider.

        access_key : typing.Optional[str]
            Access key used for requests to the provider.

        region : typing.Optional[str]
            Region used for requests to the provider.

        base_url : typing.Optional[str]
            Base URL used for requests to the provider.

        api_version : typing.Optional[str]
            API version used for requests to the provider.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Optional[typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/providers/check",
            method="POST",
            json={
                "provider_type": provider_type,
                "api_key": api_key,
                "access_key": access_key,
                "region": region,
                "base_url": base_url,
                "api_version": api_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Optional[typing.Any],
                    construct_type(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawAgentsClient, RawAgentsClient
from .types.agents_create_request_initial_message_sequence_item import AgentsCreateRequestInitialMessageSequenceItem
from .types.agents_create_response import AgentsCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AgentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAgentsClient
        """
        return self._raw_client

    def create(
        self,
        project: str,
        template_version: str,
        *,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        agent_name: typing.Optional[str] = OMIT,
        initial_message_sequence: typing.Optional[
            typing.Sequence[AgentsCreateRequestInitialMessageSequenceItem]
        ] = OMIT,
        memory_variables: typing.Optional[typing.Dict[str, str]] = OMIT,
        tool_variables: typing.Optional[typing.Dict[str, str]] = OMIT,
        identity_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsCreateResponse:
        """
        Creates an Agent or multiple Agents from a template

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        tags : typing.Optional[typing.Sequence[str]]
            The tags to assign to the agent

        agent_name : typing.Optional[str]
            The name of the agent, optional otherwise a random one will be assigned

        initial_message_sequence : typing.Optional[typing.Sequence[AgentsCreateRequestInitialMessageSequenceItem]]
            Set an initial sequence of messages, if not provided, the agent will start with the default message sequence, if an empty array is provided, the agent will start with no messages

        memory_variables : typing.Optional[typing.Dict[str, str]]
            The memory variables to assign to the agent

        tool_variables : typing.Optional[typing.Dict[str, str]]
            The tool variables to assign to the agent

        identity_ids : typing.Optional[typing.Sequence[str]]
            The identity ids to assign to the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsCreateResponse
            201

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.agents.create(
            project="project",
            template_version="template_version",
        )
        """
        _response = self._raw_client.create(
            project,
            template_version,
            tags=tags,
            agent_name=agent_name,
            initial_message_sequence=initial_message_sequence,
            memory_variables=memory_variables,
            tool_variables=tool_variables,
            identity_ids=identity_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncAgentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAgentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAgentsClient
        """
        return self._raw_client

    async def create(
        self,
        project: str,
        template_version: str,
        *,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        agent_name: typing.Optional[str] = OMIT,
        initial_message_sequence: typing.Optional[
            typing.Sequence[AgentsCreateRequestInitialMessageSequenceItem]
        ] = OMIT,
        memory_variables: typing.Optional[typing.Dict[str, str]] = OMIT,
        tool_variables: typing.Optional[typing.Dict[str, str]] = OMIT,
        identity_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentsCreateResponse:
        """
        Creates an Agent or multiple Agents from a template

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        tags : typing.Optional[typing.Sequence[str]]
            The tags to assign to the agent

        agent_name : typing.Optional[str]
            The name of the agent, optional otherwise a random one will be assigned

        initial_message_sequence : typing.Optional[typing.Sequence[AgentsCreateRequestInitialMessageSequenceItem]]
            Set an initial sequence of messages, if not provided, the agent will start with the default message sequence, if an empty array is provided, the agent will start with no messages

        memory_variables : typing.Optional[typing.Dict[str, str]]
            The memory variables to assign to the agent

        tool_variables : typing.Optional[typing.Dict[str, str]]
            The tool variables to assign to the agent

        identity_ids : typing.Optional[typing.Sequence[str]]
            The identity ids to assign to the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentsCreateResponse
            201

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.agents.create(
                project="project",
                template_version="template_version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            project,
            template_version,
            tags=tags,
            agent_name=agent_name,
            initial_message_sequence=initial_message_sequence,
            memory_variables=memory_variables,
            tool_variables=tool_variables,
            identity_ids=identity_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ... import core
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.duplicate_file_handling import DuplicateFileHandling
from ...types.file_metadata import FileMetadata
from .raw_client import AsyncRawFilesClient, RawFilesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFilesClient
        """
        return self._raw_client

    def upload(
        self,
        source_id: str,
        *,
        file: core.File,
        duplicate_handling: typing.Optional[DuplicateFileHandling] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileMetadata:
        """
        Upload a file to a data source.

        Parameters
        ----------
        source_id : str

        file : core.File
            See core.File for more documentation

        duplicate_handling : typing.Optional[DuplicateFileHandling]
            How to handle duplicate filenames

        name : typing.Optional[str]
            Optional custom name to override the uploaded file's name

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileMetadata
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.sources.files.upload(
            source_id="source_id",
        )
        """
        _response = self._raw_client.upload(
            source_id, file=file, duplicate_handling=duplicate_handling, name=name, request_options=request_options
        )
        return _response.data

    def list(
        self,
        source_id: str,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        include_content: typing.Optional[bool] = None,
        check_status_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[FileMetadata]:
        """
        List paginated files associated with a data source.

        Parameters
        ----------
        source_id : str

        limit : typing.Optional[int]
            Number of files to return

        after : typing.Optional[str]
            Pagination cursor to fetch the next set of results

        include_content : typing.Optional[bool]
            Whether to include full file content

        check_status_updates : typing.Optional[bool]
            Whether to check and update file processing status (from the vector db service). If False, will not fetch and update the status, which may lead to performance gains.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[FileMetadata]
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.sources.files.list(
            source_id="source_id",
        )
        """
        _response = self._raw_client.list(
            source_id,
            limit=limit,
            after=after,
            include_content=include_content,
            check_status_updates=check_status_updates,
            request_options=request_options,
        )
        return _response.data

    def delete(self, source_id: str, file_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a data source.

        Parameters
        ----------
        source_id : str

        file_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.sources.files.delete(
            source_id="source_id",
            file_id="file_id",
        )
        """
        _response = self._raw_client.delete(source_id, file_id, request_options=request_options)
        return _response.data


class AsyncFilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFilesClient
        """
        return self._raw_client

    async def upload(
        self,
        source_id: str,
        *,
        file: core.File,
        duplicate_handling: typing.Optional[DuplicateFileHandling] = None,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileMetadata:
        """
        Upload a file to a data source.

        Parameters
        ----------
        source_id : str

        file : core.File
            See core.File for more documentation

        duplicate_handling : typing.Optional[DuplicateFileHandling]
            How to handle duplicate filenames

        name : typing.Optional[str]
            Optional custom name to override the uploaded file's name

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileMetadata
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sources.files.upload(
                source_id="source_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload(
            source_id, file=file, duplicate_handling=duplicate_handling, name=name, request_options=request_options
        )
        return _response.data

    async def list(
        self,
        source_id: str,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        include_content: typing.Optional[bool] = None,
        check_status_updates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[FileMetadata]:
        """
        List paginated files associated with a data source.

        Parameters
        ----------
        source_id : str

        limit : typing.Optional[int]
            Number of files to return

        after : typing.Optional[str]
            Pagination cursor to fetch the next set of results

        include_content : typing.Optional[bool]
            Whether to include full file content

        check_status_updates : typing.Optional[bool]
            Whether to check and update file processing status (from the vector db service). If False, will not fetch and update the status, which may lead to performance gains.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[FileMetadata]
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sources.files.list(
                source_id="source_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            source_id,
            limit=limit,
            after=after,
            include_content=include_content,
            check_status_updates=check_status_updates,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, source_id: str, file_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a data source.

        Parameters
        ----------
        source_id : str

        file_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sources.files.delete(
                source_id="source_id",
                file_id="file_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(source_id, file_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_custom_tool_param import ChatCompletionCustomToolParam
from .chat_completion_function_tool_param import ChatCompletionFunctionToolParam

CompletionCreateParamsNonStreamingToolsItem = typing.Union[
    ChatCompletionFunctionToolParam, ChatCompletionCustomToolParam
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .agents.client import AgentsClient, AsyncAgentsClient
from .raw_client import AsyncRawTemplatesClient, RawTemplatesClient
from .types.templates_create_template_response import TemplatesCreateTemplateResponse
from .types.templates_delete_template_response import TemplatesDeleteTemplateResponse
from .types.templates_fork_template_response import TemplatesForkTemplateResponse
from .types.templates_get_template_snapshot_response import TemplatesGetTemplateSnapshotResponse
from .types.templates_list_request_sort_by import TemplatesListRequestSortBy
from .types.templates_list_response import TemplatesListResponse
from .types.templates_list_template_versions_response import TemplatesListTemplateVersionsResponse
from .types.templates_rename_template_response import TemplatesRenameTemplateResponse
from .types.templates_save_template_version_response import TemplatesSaveTemplateVersionResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTemplatesClient(client_wrapper=client_wrapper)
        self.agents = AgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTemplatesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        offset: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        template_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        project_slug: typing.Optional[str] = None,
        project_id: typing.Optional[str] = None,
        sort_by: typing.Optional[TemplatesListRequestSortBy] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesListResponse:
        """
        List all templates

        Parameters
        ----------
        offset : typing.Optional[str]

        limit : typing.Optional[str]

        template_id : typing.Optional[str]

        name : typing.Optional[str]

        search : typing.Optional[str]

        project_slug : typing.Optional[str]

        project_id : typing.Optional[str]

        sort_by : typing.Optional[TemplatesListRequestSortBy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesListResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.list()
        """
        _response = self._raw_client.list(
            offset=offset,
            limit=limit,
            template_id=template_id,
            name=name,
            search=search,
            project_slug=project_slug,
            project_id=project_id,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def savetemplateversion(
        self,
        project: str,
        template_name: str,
        *,
        preserve_environment_variables_on_migration: typing.Optional[bool] = OMIT,
        preserve_core_memories_on_migration: typing.Optional[bool] = OMIT,
        migrate_agents: typing.Optional[bool] = OMIT,
        message: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesSaveTemplateVersionResponse:
        """
        Saves the current version of the template as a new version

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The template version, formatted as {template-name}, any version appended will be ignored

        preserve_environment_variables_on_migration : typing.Optional[bool]
            If true, the environment variables will be preserved in the template version when migrating agents

        preserve_core_memories_on_migration : typing.Optional[bool]
            If true, the core memories will be preserved in the template version when migrating agents

        migrate_agents : typing.Optional[bool]
            If true, existing agents attached to this template will be migrated to the new template version

        message : typing.Optional[str]
            A message to describe the changes made in this template version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesSaveTemplateVersionResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.savetemplateversion(
            project="project",
            template_name="template_name",
        )
        """
        _response = self._raw_client.savetemplateversion(
            project,
            template_name,
            preserve_environment_variables_on_migration=preserve_environment_variables_on_migration,
            preserve_core_memories_on_migration=preserve_core_memories_on_migration,
            migrate_agents=migrate_agents,
            message=message,
            request_options=request_options,
        )
        return _response.data

    def deletetemplate(
        self, project: str, template_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplatesDeleteTemplateResponse:
        """
        Deletes all versions of a template with the specified name

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The template name (without version)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesDeleteTemplateResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.deletetemplate(
            project="project",
            template_name="template_name",
        )
        """
        _response = self._raw_client.deletetemplate(project, template_name, request_options=request_options)
        return _response.data

    def gettemplatesnapshot(
        self, project: str, template_version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplatesGetTemplateSnapshotResponse:
        """
        Get a snapshot of the template version, this will return the template state at a specific version

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesGetTemplateSnapshotResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.gettemplatesnapshot(
            project="project",
            template_version="template_version",
        )
        """
        _response = self._raw_client.gettemplatesnapshot(project, template_version, request_options=request_options)
        return _response.data

    def forktemplate(
        self,
        project: str,
        template_version: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesForkTemplateResponse:
        """
        Forks a template version into a new template

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        name : typing.Optional[str]
            Optional custom name for the forked template. If not provided, a random name will be generated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesForkTemplateResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.forktemplate(
            project="project",
            template_version="template_version",
        )
        """
        _response = self._raw_client.forktemplate(project, template_version, name=name, request_options=request_options)
        return _response.data

    def createtemplate(
        self,
        project: str,
        *,
        agent_id: str,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesCreateTemplateResponse:
        """
        Creates a new template from an existing agent

        Parameters
        ----------
        project : str
            The project slug

        agent_id : str
            The ID of the agent to use as a template, can be from any project

        name : typing.Optional[str]
            Optional custom name for the template. If not provided, a random name will be generated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesCreateTemplateResponse
            201

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.createtemplate(
            project="project",
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.createtemplate(
            project, agent_id=agent_id, name=name, request_options=request_options
        )
        return _response.data

    def renametemplate(
        self,
        project: str,
        template_name: str,
        *,
        new_name: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesRenameTemplateResponse:
        """
        Renames all versions of a template with the specified name. Versions are automatically stripped from the current template name if accidentally included.

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The current template name (version will be automatically stripped if included)

        new_name : str
            The new name for the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesRenameTemplateResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.renametemplate(
            project="project",
            template_name="template_name",
            new_name="new_name",
        )
        """
        _response = self._raw_client.renametemplate(
            project, template_name, new_name=new_name, request_options=request_options
        )
        return _response.data

    def listtemplateversions(
        self,
        project_slug: str,
        name: str,
        *,
        offset: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesListTemplateVersionsResponse:
        """
        List all versions of a specific template

        Parameters
        ----------
        project_slug : str
            The project slug

        name : str
            The template name (without version)

        offset : typing.Optional[str]

        limit : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesListTemplateVersionsResponse
            200

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )
        client.templates.listtemplateversions(
            project_slug="project_slug",
            name="name",
        )
        """
        _response = self._raw_client.listtemplateversions(
            project_slug, name, offset=offset, limit=limit, request_options=request_options
        )
        return _response.data


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTemplatesClient(client_wrapper=client_wrapper)
        self.agents = AsyncAgentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTemplatesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        offset: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        template_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        project_slug: typing.Optional[str] = None,
        project_id: typing.Optional[str] = None,
        sort_by: typing.Optional[TemplatesListRequestSortBy] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesListResponse:
        """
        List all templates

        Parameters
        ----------
        offset : typing.Optional[str]

        limit : typing.Optional[str]

        template_id : typing.Optional[str]

        name : typing.Optional[str]

        search : typing.Optional[str]

        project_slug : typing.Optional[str]

        project_id : typing.Optional[str]

        sort_by : typing.Optional[TemplatesListRequestSortBy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesListResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            offset=offset,
            limit=limit,
            template_id=template_id,
            name=name,
            search=search,
            project_slug=project_slug,
            project_id=project_id,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def savetemplateversion(
        self,
        project: str,
        template_name: str,
        *,
        preserve_environment_variables_on_migration: typing.Optional[bool] = OMIT,
        preserve_core_memories_on_migration: typing.Optional[bool] = OMIT,
        migrate_agents: typing.Optional[bool] = OMIT,
        message: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesSaveTemplateVersionResponse:
        """
        Saves the current version of the template as a new version

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The template version, formatted as {template-name}, any version appended will be ignored

        preserve_environment_variables_on_migration : typing.Optional[bool]
            If true, the environment variables will be preserved in the template version when migrating agents

        preserve_core_memories_on_migration : typing.Optional[bool]
            If true, the core memories will be preserved in the template version when migrating agents

        migrate_agents : typing.Optional[bool]
            If true, existing agents attached to this template will be migrated to the new template version

        message : typing.Optional[str]
            A message to describe the changes made in this template version

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesSaveTemplateVersionResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.savetemplateversion(
                project="project",
                template_name="template_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.savetemplateversion(
            project,
            template_name,
            preserve_environment_variables_on_migration=preserve_environment_variables_on_migration,
            preserve_core_memories_on_migration=preserve_core_memories_on_migration,
            migrate_agents=migrate_agents,
            message=message,
            request_options=request_options,
        )
        return _response.data

    async def deletetemplate(
        self, project: str, template_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplatesDeleteTemplateResponse:
        """
        Deletes all versions of a template with the specified name

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The template name (without version)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesDeleteTemplateResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.deletetemplate(
                project="project",
                template_name="template_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deletetemplate(project, template_name, request_options=request_options)
        return _response.data

    async def gettemplatesnapshot(
        self, project: str, template_version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplatesGetTemplateSnapshotResponse:
        """
        Get a snapshot of the template version, this will return the template state at a specific version

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesGetTemplateSnapshotResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.gettemplatesnapshot(
                project="project",
                template_version="template_version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.gettemplatesnapshot(
            project, template_version, request_options=request_options
        )
        return _response.data

    async def forktemplate(
        self,
        project: str,
        template_version: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesForkTemplateResponse:
        """
        Forks a template version into a new template

        Parameters
        ----------
        project : str
            The project slug

        template_version : str
            The template version, formatted as {template-name}:{version-number} or {template-name}:latest

        name : typing.Optional[str]
            Optional custom name for the forked template. If not provided, a random name will be generated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesForkTemplateResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.forktemplate(
                project="project",
                template_version="template_version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.forktemplate(
            project, template_version, name=name, request_options=request_options
        )
        return _response.data

    async def createtemplate(
        self,
        project: str,
        *,
        agent_id: str,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesCreateTemplateResponse:
        """
        Creates a new template from an existing agent

        Parameters
        ----------
        project : str
            The project slug

        agent_id : str
            The ID of the agent to use as a template, can be from any project

        name : typing.Optional[str]
            Optional custom name for the template. If not provided, a random name will be generated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesCreateTemplateResponse
            201

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.createtemplate(
                project="project",
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.createtemplate(
            project, agent_id=agent_id, name=name, request_options=request_options
        )
        return _response.data

    async def renametemplate(
        self,
        project: str,
        template_name: str,
        *,
        new_name: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesRenameTemplateResponse:
        """
        Renames all versions of a template with the specified name. Versions are automatically stripped from the current template name if accidentally included.

        Parameters
        ----------
        project : str
            The project slug

        template_name : str
            The current template name (version will be automatically stripped if included)

        new_name : str
            The new name for the template

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesRenameTemplateResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.renametemplate(
                project="project",
                template_name="template_name",
                new_name="new_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.renametemplate(
            project, template_name, new_name=new_name, request_options=request_options
        )
        return _response.data

    async def listtemplateversions(
        self,
        project_slug: str,
        name: str,
        *,
        offset: typing.Optional[str] = None,
        limit: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplatesListTemplateVersionsResponse:
        """
        List all versions of a specific template

        Parameters
        ----------
        project_slug : str
            The project slug

        name : str
            The template name (without version)

        offset : typing.Optional[str]

        limit : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplatesListTemplateVersionsResponse
            200

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            project="YOUR_PROJECT",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.templates.listtemplateversions(
                project_slug="project_slug",
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.listtemplateversions(
            project_slug, name, offset=offset, limit=limit, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_completion_message_custom_tool_call_param import ChatCompletionMessageCustomToolCallParam
from .chat_completion_message_function_tool_call_param import ChatCompletionMessageFunctionToolCallParam

ChatCompletionAssistantMessageParamToolCallsItem = typing.Union[
    ChatCompletionMessageFunctionToolCallParam, ChatCompletionMessageCustomToolCallParam
]
